@@ -17,6 +17,7 @@ from UM.Scene.SceneNodeDecorator import SceneNodeDecorator
 from copy import deepcopy
 import numpy
 
+
 ##  A scene node object.
 #
 #   These objects can hold a mesh and multiple children. Each node has a transformation matrix
@@ -27,7 +28,7 @@ import numpy
 #   \sa SceneNodeDecorator
 #   \todo Add unit testing
 @signalemitter
-class SceneNode():
+class SceneNode:
     class TransformSpace:
         Local = 1
         Parent = 2
@@ -39,11 +40,11 @@ class SceneNode():
     #                 Possible keywords:
     #                 - visible \type{bool} Is the SceneNode (and thus, all it's children) visible? Defaults to True
     #                 - name \type{string} Name of the SceneNode. Defaults to empty string.
-    def __init__(self, parent = None, **kwargs):
+    def __init__(self, parent: Optional["SceneNode"] = None, **kwargs):
         super().__init__()  # Call super to make multiple inheritance work.
 
         self._children = []     # type: List[SceneNode]
-        self._mesh_data = None  # type: MeshData
+        self._mesh_data = None  # type: Optional[MeshData]
 
         # Local transformation (from parent to local)
         self._transformation = Matrix()  # type: Matrix
@@ -125,6 +126,9 @@ class SceneNode():
     def getMirror(self) -> Vector:
         return self._mirror
 
+    def setMirror(self, vector) -> Vector:
+        self._mirror = vector
+
     ##  Get the MeshData of the bounding box
     #   \returns \type{MeshData} Bounding box mesh.
     def getBoundingBoxMesh(self) -> Optional[MeshData]:
@@ -296,7 +300,7 @@ class SceneNode():
     #          If this node is a group, it will recursively concatenate all child nodes/objects.
     #   \returns MeshData
     def getMeshDataTransformed(self) -> Optional[MeshData]:
-        return MeshData(vertices = self.getMeshDataTransformedVertices())
+        return MeshData(vertices = self.getMeshDataTransformedVertices(), normals = self.getMeshDataTransformedNormals())
 
     ##  \brief Get the transformed vertices from this scene node/object, based on the transformation of scene nodes wrt root.
     #          If this node is a group, it will recursively concatenate all child nodes/objects.
@@ -314,6 +318,22 @@ class SceneNode():
             transformed_vertices = self._mesh_data.getTransformed(self.getWorldTransformation()).getVertices()
         return transformed_vertices
 
+    ##  \brief Get the transformed normals from this scene node/object, based on the transformation of scene nodes wrt root.
+    #          If this node is a group, it will recursively concatenate all child nodes/objects.
+    #   \return numpy.ndarray
+    def getMeshDataTransformedNormals(self) -> numpy.ndarray:
+        transformed_normals = None
+        if self.callDecoration("isGroup"):
+            for child in self._children:
+                tv = child.getMeshDataTransformedNormals()
+                if transformed_normals is None:
+                    transformed_normals = tv
+                else:
+                    transformed_normals = numpy.concatenate((transformed_normals, tv), axis = 0)
+        else:
+            transformed_normals = self._mesh_data.getTransformed(self.getWorldTransformation()).getNormals()
+        return transformed_normals
+
     ##  \brief Set the mesh of this node/object
     #   \param mesh_data MeshData object
     def setMeshData(self, mesh_data: Optional[MeshData]):
@@ -626,10 +646,10 @@ class SceneNode():
     def getShear(self) -> Vector:
         return self._shear
 
-    def getSetting(self, key, default_value):
+    def getSetting(self, key: str, default_value: str = "") -> str:
         return self._settings.get(key, default_value)
 
-    def setSetting(self, key, value):
+    def setSetting(self, key: str, value: str):
         self._settings[key] = value
 
     ##  private:
@@ -687,3 +707,8 @@ class SceneNode():
             else:
                 aabb = aabb + child.getBoundingBox()
         self._aabb = aabb
+
+    ##  String output for debugging.
+    def __str__(self):
+        name = self._name if self._name != "" else hex(id(self))
+        return "<" + self.__class__.__qualname__ + " object: '" + name + "'>"
\ No newline at end of file
